<?php

namespace Aboutgoods\RabbitMQ;

use PhpAmqpLib\Channel\AMQPChannel;
use PhpAmqpLib\Connection\AMQPStreamConnection;
use PhpAmqpLib\Message\AMQPMessage;

class Client
{
    /**
     * @var AMQPStreamConnection
     */
    private $connection;
    /**
     * @var AMQPChannel
     */
    private $channel;
    private $channelName;
    private $address;
    private $port;
    private $login;
    private $password;

    public function __construct($address = "localhost", $port = 5672, $login = "guest", $password = "guest")
    {
        $this->address = $address;
        $this->port = $port;
        $this->login = $login;
        $this->password = $password;
    }

    public function connect()
    {
        $this->connection = new AMQPStreamConnection($this->address, $this->port, $this->login, $this->password);
    }

    public function setConnection($connection)
    {
        if ($connection instanceof AMQPStreamConnection) {
            $this->connection = $connection;
        }
    }

    public function connectToChannel(
        $queue = '',
        $passive = false,
        $durable = false,
        $exclusive = false,
        $auto_delete = true
    ) {
        $this->channel = $this->connection->channel();
        $this->channelName = $queue;
        return $this->channel->queue_declare($queue, $passive, $durable, $exclusive, $auto_delete);
    }

    public function send($content)
    {
        $msg = new AMQPMessage($content, ['delivery_mode' => AMQPMessage::DELIVERY_MODE_PERSISTENT]);
        $this->channel->basic_publish($msg, '', $this->channelName);
        return true;
    }

    public function __destruct()
    {
        $this->channel->close();
        $this->connection->close();
    }
}